@@ -1,6 +1,4 @@
-"""GNUmed clinical narrative business object.
-
-"""
+"""GNUmed clinical narrative business object."""
 #============================================================
 __version__ = "$Revision: 1.45 $"
 __author__ = "Carlos Moro <cfmoro1976@yahoo.es>, Karsten Hilbert <Karsten.Hilbert@gmx.net>"
@@ -11,13 +9,15 @@ import sys, logging
 
 if __name__ == '__main__':
 	sys.path.insert(0, '../../')
-from Gnumed.pycommon import gmPG2, gmExceptions, gmBusinessDBObject, gmTools
+from Gnumed.pycommon import gmPG2, gmExceptions, gmBusinessDBObject, gmTools, gmDispatcher, gmHooks
+
 
 try:
 	_('dummy-no-need-to-translate-but-make-epydoc-happy')
 except NameError:
 	_ = lambda x:x
 
+
 _log = logging.getLogger('gm.emr')
 _log.info(__version__)
 
@@ -47,6 +47,14 @@ l10n2soap_cat = {
 	#_('soap_ADMIN').replace(u'soap_', u''): None
 	gmTools.u_ellipsis: None
 }
+
+#============================================================
+def _on_soap_modified():
+	"""Always relates to the active patient."""
+	gmHooks.run_hook_script(hook = u'after_soap_modified')
+
+gmDispatcher.connect(_on_soap_modified, u'clin_narrative_mod_db')
+
 #============================================================
 class cDiag(gmBusinessDBObject.cBusinessDBObject):
 	"""Represents one real diagnosis.
@@ -256,6 +264,12 @@ def delete_clin_narrative(narrative=None):
 #------------------------------------------------------------
 if __name__ == '__main__':
 
+	if len(sys.argv) < 2:
+		sys.exit()
+
+	if sys.argv[1] != 'test':
+		sys.exit()
+
 	from Gnumed.pycommon import gmI18N
 	gmI18N.activate_locale()
 	gmI18N.install_domain(domain = 'gnumed')
@@ -302,147 +316,4 @@ if __name__ == '__main__':
 	test_narrative()
 
 #============================================================
-# $Log: gmClinNarrative.py,v $
-# Revision 1.45  2010-01-31 16:30:20  ncq
-# - cleanup
-#
-# Revision 1.44  2009/12/26 19:05:08  ncq
-# - improved comment
-#
-# Revision 1.43  2009/11/13 20:46:49  ncq
-# - limit EMR search to 1000 results to remain sane
-#
-# Revision 1.42  2009/11/08 20:42:00  ncq
-# - search across EMRs
-#
-# Revision 1.41  2009/06/29 14:50:30  ncq
-# - use ellipsis for administrational soap rows
-#
-# Revision 1.40  2009/05/13 12:17:43  ncq
-# - enable setting of fk_encounter
-#
-# Revision 1.39  2009/04/16 12:46:26  ncq
-# - episode/encounter patient consistency check now moved to database
-#
-# Revision 1.38  2009/01/15 11:31:00  ncq
-# - cleanup
-#
-# Revision 1.37  2008/12/27 15:49:21  ncq
-# - raise exception on integrity problems in create_narrative
-#
-# Revision 1.36  2008/12/18 21:25:56  ncq
-# - add format() to cClinNarrative
-#
-# Revision 1.35  2008/08/15 15:55:14  ncq
-# - cleanup
-#
-# Revision 1.34  2008/05/07 15:15:15  ncq
-# - use replace() rather than strip() to remove suffixes
-#
-# Revision 1.33  2008/04/22 21:11:05  ncq
-# - define _() for testing
-#
-# Revision 1.32  2008/04/11 12:20:16  ncq
-# - soap_cat2l10n_str
-#
-# Revision 1.31  2008/01/30 13:34:49  ncq
-# - switch to std lib logging
-#
-# Revision 1.30  2008/01/22 22:02:29  ncq
-# - add_coded_term -> add_coded_phrase
-#
-# Revision 1.29  2008/01/07 11:40:21  ncq
-# - fix faulty comparison
-#
-# Revision 1.28  2008/01/06 08:08:25  ncq
-# - check for duplicate narrative before insertion
-#
-# Revision 1.27  2007/11/05 12:09:29  ncq
-# - support admin soap type
-#
-# Revision 1.26  2007/09/10 12:31:55  ncq
-# - improve test suite
-#
-# Revision 1.25  2007/09/07 22:36:44  ncq
-# - soap_cat2l10n and back
-#
-# Revision 1.24  2006/11/20 15:55:12  ncq
-# - gmPG2.run_ro_queries *always* returns (rows, idx) so be aware of that
-#
-# Revision 1.23  2006/10/08 15:02:14  ncq
-# - convert to gmPG2
-# - convert to cBusinessDBObject
-#
-# Revision 1.22  2006/07/19 20:25:00  ncq
-# - gmPyCompat.py is history
-#
-# Revision 1.21  2006/05/06 18:51:55  ncq
-# - remove comment
-#
-# Revision 1.20  2005/11/27 12:44:57  ncq
-# - clinical tables are in schema "clin" now
-#
-# Revision 1.19  2005/10/10 18:27:34  ncq
-# - v_pat_narrative already HAS .provider
-#
-# Revision 1.18  2005/10/08 12:33:09  sjtan
-# tree can be updated now without refetching entire cache; done by passing emr object to create_xxxx methods and calling emr.update_cache(key,obj);refresh_historical_tree non-destructively checks for changes and removes removed nodes and adds them if cache mismatch.
-#
-# Revision 1.17  2005/09/19 16:32:02  ncq
-# - remove is_rfe/is_aoe/cRFE/cAOE
-#
-# Revision 1.16  2005/06/09 21:29:16  ncq
-# - added missing s in %()s
-#
-# Revision 1.15  2005/05/17 08:00:09  ncq
-# - in create_narrative() ignore empty narrative
-#
-# Revision 1.14  2005/04/11 17:53:47  ncq
-# - id_patient -> pk_patient fix
-#
-# Revision 1.13  2005/04/08 13:27:54  ncq
-# - adapt get_codes()
-#
-# Revision 1.12  2005/01/31 09:21:48  ncq
-# - use commit2()
-# - add delete_clin_narrative()
-#
-# Revision 1.11  2005/01/02 19:55:30  ncq
-# - don't need _xmins_refetch_col_pos anymore
-#
-# Revision 1.10  2004/12/20 16:45:49  ncq
-# - gmBusinessDBObject now requires refetching of XMIN after save_payload
-#
-# Revision 1.9  2004/11/03 22:32:34  ncq
-# - support _cmds_lock_rows_for_update in business object base class
-#
-# Revision 1.8  2004/09/25 13:26:35  ncq
-# - is_significant -> clinically_relevant
-#
-# Revision 1.7  2004/08/11 09:42:50  ncq
-# - point clin_narrative VO to v_pat_narrative
-# - robustify by applying lower() to soap_cat on insert/update
-#
-# Revision 1.6  2004/07/25 23:23:39  ncq
-# - Carlos made cAOE.get_diagnosis() return a cDiag instead of a list
-#
-# Revision 1.5	2004/07/14 09:10:21	 ncq
-# - Carlos' relentless work brings us get_codes(),
-#	get_possible_codes() and adjustions for the fact
-#	that we can now code any soap row
-#
-# Revision 1.4	2004/07/07 15:05:51	 ncq
-# - syntax fixes by Carlos
-# - get_codes(), get_possible_codes()
-# - talk to the right views
-#
-# Revision 1.3	2004/07/06 00:09:19	 ncq
-# - Carlos added create_clin_narrative(), cDiag, cNarrative, and unit tests - nice work !
-#
-# Revision 1.2	2004/07/05 10:24:46	 ncq
-# - use v_pat_rfe/aoe, by Carlos
-#
-# Revision 1.1	2004/07/04 13:24:31	 ncq
-# - add cRFE/cAOE
-# - use in get_rfes(), get_aoes()
-#
+
