@@ -187,13 +187,16 @@ class cHealthIssue(gmBusinessDBObject.cBusinessDBObject):
 		if self._payload[self._idx['age_noted']] is None:
 			return u'<???>'
 
+		# since we've already got an interval we are bound to use it,
+		# further transformation will only introduce more errors,
+		# later we can improve this deeper inside
 		return gmDateTime.format_interval_medically(self._payload[self._idx['age_noted']])
 	#--------------------------------------------------------
 	def _get_laterality_description(self):
 		try:
 			return laterality2str[self._payload[self._idx['laterality']]]
 		except KeyError:
-			return u'??'
+			return u'<???>'
 
 	laterality_description = property(_get_laterality_description, lambda x:x)
 	#--------------------------------------------------------
@@ -219,7 +222,7 @@ class cHealthIssue(gmBusinessDBObject.cBusinessDBObject):
 			self._payload[self._idx['description']],
 			u'\u00AB',
 			gmTools.coalesce (
-				initial = self.laterality_description,		#laterality2str[self._payload[self._idx['laterality']]],
+				initial = self.laterality_description,
 				instead = u'',
 				template_initial = u' (%s)',
 				none_equivalents = [None, u'', u'?']
@@ -343,22 +346,32 @@ class cHealthIssue(gmBusinessDBObject.cBusinessDBObject):
 
 		epis = self.get_episodes()
 		if len(epis) > 0:
+			epi_pks = [ e['pk_episode'] for e in epis ]
+
 			# documents
 			doc_folder = patient.get_document_folder()
-			docs = doc_folder.get_documents(episodes = [ e['pk_episode'] for e in epis ])
-
+			docs = doc_folder.get_documents(episodes = epi_pks)
 			if len(docs) > 0:
 				lines.append(u'')
 				lines.append(_('Documents: %s') % len(docs))
 			del docs
 
 			# test results
-			tests = emr.get_test_results_by_date(episodes = [ e['pk_episode'] for e in epis ])
+			tests = emr.get_test_results_by_date(episodes = epi_pks)
 			if len(tests) > 0:
 				lines.append(u'')
 				lines.append(_('Measurements and Results: %s') % len(tests))
 			del tests
 
+			# vaccinations
+			vaccs = emr.get_vaccinations(episodes = epi_pks)
+			if len(vaccs) > 0:
+				lines.append(u'')
+				lines.append(_('Vaccinations:'))
+			for vacc in vaccs:
+				lines.extend(vacc.format(with_reaction = True))
+			del vaccs
+
 		del epis
 
 		left_margin = u' ' * left_margin
@@ -720,6 +733,22 @@ from (
 			))
 		del tests
 
+		# vaccinations
+		vaccs = emr.get_vaccinations(episodes = [ self._payload[self._idx['pk_episode']] ])
+
+		if len(vaccs) > 0:
+			lines.append(u'')
+			lines.append(_('Vaccinations:'))
+
+		for vacc in vaccs:
+			lines.extend(vacc.format (
+				with_indications = True,
+				with_comment = True,
+				with_reaction = True,
+				date_format = '%Y-%m-%d'
+			))
+		del vaccs
+
 		left_margin = u' ' * left_margin
 		eol_w_margin = u'\n%s' % left_margin
 		return left_margin + eol_w_margin.join(lines) + u'\n'
@@ -997,7 +1026,7 @@ limit 1
 
 		return lines
 	#--------------------------------------------------------
-	def format(self, episodes=None, with_soap=False, left_margin=0, patient=None, issues=None, with_docs=True, with_tests=True, fancy_header=True):
+	def format(self, episodes=None, with_soap=False, left_margin=0, patient=None, issues=None, with_docs=True, with_tests=True, fancy_header=True, with_vaccinations=True):
 
 		lines = []
 
@@ -1078,6 +1107,27 @@ limit 1
 
 			del tests
 
+		# vaccinations
+		if with_vaccinations:
+			vaccs = emr.get_vaccinations (
+				episodes = episodes,
+				encounters = [ self._payload[self._idx['pk_encounter']] ]
+			)
+
+			if len(vaccs) > 0:
+				lines.append(u'')
+				lines.append(_('Vaccinations:'))
+
+			for vacc in vaccs:
+				lines.extend(vacc.format (
+					with_indications = True,
+					with_comment = True,
+					with_reaction = True,
+					date_format = '%Y-%m-%d'
+				))
+			del vaccs
+
+		# documents
 		if with_docs:
 			doc_folder = patient.get_document_folder()
 			docs = doc_folder.get_documents (
@@ -1101,19 +1151,6 @@ limit 1
 
 		eol_w_margin = u'\n%s' % (u' ' * left_margin)
 		return u'%s\n' % eol_w_margin.join(lines)
-
-		# special items (vaccinations, ...)
-
-#        try:
- #               filtered_items.extend(emr.get_vaccinations(
-  #                  since=self.__constraints['since'],
-   #                 until=self.__constraints['until'],
-    #                encounters=self.__constraints['encounters'],
-     #               episodes=self.__constraints['episodes'],
-      #              issues=self.__constraints['issues']))
-       # except:
-        #        _log.error("vaccination error? outside regime")
-
 #-----------------------------------------------------------
 def create_encounter(fk_patient=None, fk_location=-1, enc_type=None):
 	"""Creates a new encounter for a patient.
@@ -1391,18 +1428,24 @@ class cPerformedProcedure(gmBusinessDBObject.cBusinessDBObject):
 
 	_cmd_fetch_payload = u"select * from clin.v_pat_procedures where pk_procedure = %s"
 	_cmds_store_payload = [
-		u"""update clin.procedure set
+		u"""UPDATE clin.procedure SET
 				clin_when = %(clin_when)s,
-				clin_where = gm.nullify_empty_string(%(clin_where)s),
+				clin_where = NULLIF (
+					COALESCE (
+						%(pk_hospital_stay)s::TEXT,
+						gm.nullify_empty_string(%(clin_where)s)
+					),
+					%(pk_hospital_stay)s::TEXT
+				),
 				narrative = gm.nullify_empty_string(%(performed_procedure)s),
 				fk_hospital_stay = %(pk_hospital_stay)s,
 				fk_episode = %(pk_episode)s,
 				fk_encounter = %(pk_encounter)s
-			where
-				pk = %(pk_procedure)s and
+			WHERE
+				pk = %(pk_procedure)s AND
 				xmin = %(xmin_procedure)s
-		""",
-		u"""select xmin_procedure from clin.v_pat_procedures where pk_procedure = %(pk_procedure)s"""
+			RETURNING xmin as xmin_procedure"""
+#		,u"""select xmin_procedure from clin.v_pat_procedures where pk_procedure = %(pk_procedure)s"""
 	]
 	_updatable_fields = [
 		'clin_when',
@@ -1453,20 +1496,20 @@ def create_performed_procedure(encounter=None, episode=None, location=None, hosp
 
 	queries = [{
 		'cmd': u"""
-insert into clin.procedure (
-	fk_encounter,
-	fk_episode,
-	clin_where,
-	fk_hospital_stay,
-	narrative
-) values (
-	%(enc)s,
-	%(epi)s,
-	gm.nullify_empty_string(%(loc)s),
-	%(stay)s,
-	%(proc)s
-)
-returning pk""",
+			INSERT INTO clin.procedure (
+				fk_encounter,
+				fk_episode,
+				clin_where,
+				fk_hospital_stay,
+				narrative
+			) VALUES (
+				%(enc)s,
+				%(epi)s,
+				gm.nullify_empty_string(%(loc)s),
+				%(stay)s,
+				gm.nullify_empty_string(%(proc)s)
+			)
+			RETURNING pk""",
 		'args': {'enc': encounter, 'epi': episode, 'loc': location, 'stay': hospital_stay, 'proc': procedure}
 	}]
 
