@@ -10,7 +10,9 @@ __version__ = "$Revision: 1.79 $"
 __author__ ="Ian Haywood <ihaywood@gnu.org>, karsten.hilbert@gmx.net"
 
 
-import os, sys, time, os.path, logging, codecs, re as regex, shutil, random, platform, subprocess
+import os, sys, time, os.path, logging, codecs, re as regex
+import shutil, random, platform, subprocess
+import socket										# needed for OOo on Windows
 #, libxml2, libxslt
 
 
@@ -26,17 +28,19 @@ _log.info(__version__)
 
 #============================================================
 # this order is also used in choice boxes for the engine
-form_engine_abbrevs = [u'O', u'L', u'I']
+form_engine_abbrevs = [u'O', u'L', u'I', u'G']
 
 form_engine_names = {
 	u'O': 'OpenOffice',
 	u'L': 'LaTeX',
-	u'I': 'Image editor'
+	u'I': 'Image editor',
+	u'G': 'Gnuplot script'
 }
 
 form_engine_template_wildcards = {
 	u'O': u'*.o?t',
-	u'L': u'*.tex'
+	u'L': u'*.tex',
+	u'G': u'*.gpl'
 }
 
 # is filled in further below after each engine is defined
@@ -156,8 +160,7 @@ class cFormTemplate(gmBusinessDBObject.cBusinessDBObject):
 
 			filename = gmTools.get_unique_filename (
 				prefix = 'gm-%s-Template-' % self._payload[self._idx['engine']],
-				suffix = suffix,
-				tmp_dir = os.path.expanduser(os.path.join('~', '.gnumed', 'tmp'))
+				suffix = suffix
 			)
 
 		data_query = {
@@ -206,22 +209,27 @@ def get_form_template(name_long=None, external_version=None):
 
 	return cFormTemplate(aPK_obj = rows[0]['pk'])
 #------------------------------------------------------------
-def get_form_templates(engine=None, active_only=False):
+def get_form_templates(engine=None, active_only=False, template_types=None, excluded_types=None):
 	"""Load form templates."""
 
 	args = {'eng': engine, 'in_use': active_only}
+	where_parts = [u'1 = 1']
 
-	where_parts = []
 	if engine is not None:
 		where_parts.append(u'engine = %(eng)s')
 
 	if active_only:
-		where_parts.append(u'in_use is True')
+		where_parts.append(u'in_use IS true')
 
-	if len(where_parts) == 0:
-		cmd = u"select * from ref.v_paperwork_templates order by in_use desc, name_long"
-	else:
-		cmd = u"select * from ref.v_paperwork_templates where %s order by in_use desc, name_long" % u'and'.join(where_parts)
+	if template_types is not None:
+		args['incl_types'] = tuple(template_types)
+		where_parts.append(u'template_type IN %(incl_types)s')
+
+	if excluded_types is not None:
+		args['excl_types'] = tuple(excluded_types)
+		where_parts.append(u'template_type NOT IN %(excl_types)s')
+
+	cmd = u"SELECT * FROM ref.v_paperwork_templates WHERE %s ORDER BY in_use desc, name_long" % u'\nAND '.join(where_parts)
 
 	rows, idx = gmPG2.run_ro_queries (
 		queries = [{'cmd': cmd, 'args': args}],
@@ -643,7 +651,7 @@ class cOOoForm(cFormEngine):
 
 		path, ext = os.path.splitext(self.template_filename)
 		if ext in [r'', r'.']:
-			ext = r'.tex'
+			ext = r'.odt'
 		self.instance_filename = r'%s-instance%s' % (path, ext)
 
 #================================================================
@@ -777,12 +785,66 @@ class cLaTeXForm(cFormEngine):
 			os.remove(self.template_filename)
 		except:
 			_log.debug(u'cannot remove template file [%s]', self.template_filename)
-	#--------------------------------------------------------
-	# internal helpers
-	#--------------------------------------------------------
-
 #------------------------------------------------------------
 form_engines[u'L'] = cLaTeXForm
+#============================================================
+# Gnuplot template forms
+#------------------------------------------------------------
+class cGnuplotForm(cFormEngine):
+	"""A forms engine wrapping Gnuplot."""
+
+	#--------------------------------------------------------
+	def substitute_placeholders(self, data_source=None):
+		"""Parse the template into an instance and replace placeholders with values."""
+		pass
+	#--------------------------------------------------------
+	def edit(self):
+		"""Allow editing the instance of the template."""
+		pass
+	#--------------------------------------------------------
+	def generate_output(self, format=None):
+		"""Generate output suitable for further processing outside this class, e.g. printing.
+
+		Expects .data_filename to be set.
+		"""
+		self.conf_filename = gmTools.get_unique_filename(prefix = 'gm2gpl-', suffix = '.conf')
+		fname_file = codecs.open(self.conf_filename, 'wb', 'utf8')
+		fname_file.write('# setting the gnuplot data file\n')
+		fname_file.write("gm2gpl_datafile = '%s'\n" % self.data_filename)
+		fname_file.close()
+
+		# FIXME: cater for configurable path
+		if platform.system() == 'Windows':
+			exec_name = 'gnuplot.exe'
+		else:
+			exec_name = 'gnuplot'
+
+		args = [exec_name, '-p', self.conf_filename, self.template_filename]
+		_log.debug('plotting args: %s' % str(args))
+
+		try:
+			gp = subprocess.Popen (
+				args = args,
+				close_fds = True
+			)
+		except (OSError, ValueError, subprocess.CalledProcessError):
+			_log.exception('there was a problem executing gnuplot')
+			gmDispatcher.send(signal = u'statustext', msg = _('Error running gnuplot. Cannot plot data.'), beep = True)
+			return
+
+		gp.communicate()
+
+		return
+	#--------------------------------------------------------
+	def cleanup (self):
+		try:
+			os.remove(self.template_filename)
+			os.remove(self.conf_filename)
+			os.remove(self.data_filename)
+		except StandardError:
+			_log.exception(u'cannot remove either of script/conf/data file')
+#------------------------------------------------------------
+form_engines[u'G'] = cGnuplotForm
 #------------------------------------------------------------
 #------------------------------------------------------------
 class cIanLaTeXForm(cFormEngine):
@@ -937,10 +999,6 @@ class cXSLTFormEngine(cFormEngine):
 		self.preview()
 
 
-#=====================================================
-engines = {
-	u'L': cLaTeXForm
-}
 #=====================================================
 #class LaTeXFilter(Cheetah.Filters.Filter):
 class LaTeXFilter:
@@ -1092,7 +1150,7 @@ crap to demonstrate how it can cover multiple lines.""",
 	form.process (params)
 	form.xdvi ()
 	form.cleanup ()
-	
+
 def test_au2 ():
 	form = LaTeXForm (2, test_letter)
 	params = {'RECIPIENTNAME':'Dr. Richard Terry',
@@ -1255,7 +1313,7 @@ if __name__ == '__main__':
 	#test_de()
 
 	# OOo
-	test_init_ooo()
+	#test_init_ooo()
 	#test_ooo_connect()
 	#test_open_ooo_doc_from_srv()
 	#test_open_ooo_doc_from_letter()
@@ -1264,6 +1322,6 @@ if __name__ == '__main__':
 
 	#test_cFormTemplate()
 	#set_template_from_file()
-	#test_latex_form()
+	test_latex_form()
 
 #============================================================
