@@ -27,6 +27,45 @@ def _on_substance_intake_modified():
 
 gmDispatcher.connect(_on_substance_intake_modified, u'substance_intake_mod_db')
 
+#============================================================
+def drug2renal_insufficiency_url(search_term=None):
+
+	if search_term is None:
+		return u'http://www.dosing.de'
+
+	terms = []
+	names = []
+
+	if isinstance(search_term, cBrandedDrug):
+		if search_term['atc_code'] is not None:
+			terms.append(search_term['atc_code'])
+
+	elif isinstance(search_term, cSubstanceIntakeEntry):
+		names.append(search_term['substance'])
+		if search_term['atc_brand'] is not None:
+			terms.append(search_term['atc_brand'])
+		if search_term['atc_substance'] is not None:
+			terms.append(search_term['atc_substance'])
+
+	elif search_term is not None:
+		names.append(u'%s' % search_term)
+		terms.extend(gmATC.text2atc(text = u'%s' % search_term, fuzzy = True))
+
+	for name in names:
+		if name.endswith('e'):
+			terms.append(name[:-1])
+		else:
+			terms.append(name)
+
+	#url_template = u'http://www.google.de/#q=site%%3Adosing.de+%s'
+	#url = url_template % u'+OR+'.join(terms)
+
+	url_template = u'http://www.google.de/search?hl=de&source=hp&q=site%%3Adosing.de+%s&btnG=Google-Suche'
+	url = url_template % u'+OR+'.join(terms)
+
+	_log.debug(u'renal insufficiency URL: %s', url)
+
+	return url
 #============================================================
 # this should be in gmCoding.py
 def create_data_source(long_name=None, short_name=None, version=None, source=None, language=None):
@@ -212,17 +251,15 @@ class cDrugDataSourceInterface(object):
 #============================================================
 class cFreeDiamsInterface(cDrugDataSourceInterface):
 
-	"""http://ericmaeker.fr/FreeMedForms/di-manual/ligne_commandes.html"""
-
-	version = u'FreeDiams v0.3.0 interface'
+	version = u'FreeDiams v0.4.2 interface'
 	default_encoding = 'utf8'
-	default_dob_format = '%d/%m/%Y'
+	default_dob_format = '%Y/%m/%d'
 
 	map_gender2mf = {
 		'm': u'M',
 		'f': u'F',
-		'tf': u'F',
-		'tm': u'M',
+		'tf': u'H',
+		'tm': u'H',
 		'h': u'H'
 	}
 	#--------------------------------------------------------
@@ -236,7 +273,7 @@ class cFreeDiamsInterface(cDrugDataSourceInterface):
 	def get_data_source_version(self):
 		#> Coded. Available next release
 		#> Use --version or -version or -v
-		return u'0.3.0'
+		return u'0.4.2'
 		# ~/.freediams/config.ini: [License] -> AcceptedVersion=....
 	#--------------------------------------------------------
 	def create_data_source_entry(self):
@@ -249,11 +286,8 @@ class cFreeDiamsInterface(cDrugDataSourceInterface):
 		)
 	#--------------------------------------------------------
 	def switch_to_frontend(self, blocking=False):
-		"""	--medintux : définit une utilisation spécifique à MedinTux.
-			--exchange="xxx" : définit le fichier d'échange entre les deux applications.
-			--chrono : Chronomètres diverses fonctions du testeur d'interactions (proposé à des fins de déboggage)
-			--transmit-dosage = non documenté.
-		"""
+		"""http://ericmaeker.fr/FreeMedForms/di-manual/ligne_commandes.html"""
+
 		found, cmd = gmShellAPI.find_first_binary(binaries = [
 			self.custom_path_to_binary,
 			r'/usr/bin/freediams',
@@ -269,22 +303,16 @@ class cFreeDiamsInterface(cDrugDataSourceInterface):
 
 		# make sure csv file exists
 		open(self.__exchange_filename, 'wb').close()
-		args = u'--exchange="%s"' % self.__exchange_filename
+		#args = u'--exchange="%s" --blockpatientdatas="1"' % self.__exchange_filename
+		args = u'--blockpatientdatas="1"'
 
 		if self.patient is not None:
-
-			args += u' --patientname="%(firstnames)s %(lastnames)s"' % self.patient.get_active_name()
-
+			names = self.patient.get_active_name()
+			args += u' --patientname="%(lastnames)s, %(firstnames)s"' % names
+			args += u' --patientsurname="%(lastnames)s"' % names
 			args += u' --gender=%s' % cFreeDiamsInterface.map_gender2mf[self.patient['gender']]
-
 			if self.patient['dob'] is not None:
-				args += u' --dateofbirth="%s"' % self.patient['dob'].strftime(cFreeDiamsInterface.default_dob_format)
-
-			# FIXME: search by LOINC code and add
-			# --weight="dd" : définit le poids du patient (en kg)
-			# --size="ddd" : définit la taille du patient (en cm)
-			# --clcr="dd.d" : définit la clairance de la créatinine du patient (en ml/min)
-			# --creatinin="dd" : définit la créatininémie du patient (en mg/l)
+				args += u' --dateofbirth=%s' % self.patient['dob'].strftime(cFreeDiamsInterface.default_dob_format)
 
 		cmd = r'%s %s' % (cmd, args)
 
@@ -613,7 +641,7 @@ class cIfapInterface(cDrugDataSourceInterface):
 drug_data_source_interfaces = {
 	'Deutschland: Gelbe Liste/MMI (Windows)': cGelbeListeWindowsInterface,
 	'Deutschland: Gelbe Liste/MMI (WINE)': cGelbeListeWineInterface,
-	'France: FreeDiams': cFreeDiamsInterface
+	'FreeDiams (France, US, Canada)': cFreeDiamsInterface
 }
 #============================================================
 # substances in use across all patients
@@ -687,7 +715,7 @@ class cSubstanceIntakeEntry(gmBusinessDBObject.cBusinessDBObject):
 						when (
 							(%(is_long_term)s is False)
 								and
-							(%(duration)s IS NULL)
+							(%(duration)s is NULL)
 						) is True then null
 						else %(is_long_term)s
 					end
@@ -747,7 +775,7 @@ class cSubstanceIntakeEntry(gmBusinessDBObject.cBusinessDBObject):
 			gmTools.u_right_arrow,
 			duration,
 			self._payload[self._idx['substance']],
-			self._payload[self._idx['strength']],
+			gmTools.coalesce(self._payload[self._idx['strength']], u''),
 			self._payload[self._idx['preparation']],
 			gmTools.bool2subst(self._payload[self._idx['is_currently_active']], _('ongoing'), _('inactive'), _('?ongoing'))
 		)
@@ -770,7 +798,11 @@ class cSubstanceIntakeEntry(gmBusinessDBObject.cBusinessDBObject):
 		if self._payload[self._idx['external_code_brand']] is not None:
 			allg['substance_code'] = u'%s::::%s' % (self._payload[self._idx['external_code_type_brand']], self._payload[self._idx['external_code_brand']])
 		allg['allergene'] = self._payload[self._idx['substance']]
-		allg['generics'] = self._payload[self._idx['substance']]
+		comps = [ c['description'] for c in self.containing_drug.components ]
+		if len(comps) == 0:
+			allg['generics'] = self._payload[self._idx['substance']]
+		else:
+			allg['generics'] = u'; '.join(comps)
 
 		allg.save()
 		return allg
@@ -872,6 +904,119 @@ returning pk
 def delete_substance_intake(substance=None):
 	cmd = u'delete from clin.substance_intake where pk = %(pk)s'
 	gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': {'pk': substance}}])
+#------------------------------------------------------------
+def format_substance_intake_notes(emr=None, output_format=u'latex', table_type=u'by-brand'):
+
+	tex =  u'\n{\\small\n'
+	tex += u'\\noindent %s\n' % _('Additional notes')
+	tex += u'\n'
+	tex += u'\\noindent \\begin{tabular}{|l|l|l|l|}\n'
+	tex += u'\\hline\n'
+	tex += u'%s & %s & %s & \\\\ \n' % (_('Substance'), _('Strength'), _('Brand'))
+	tex += u'\\hline\n'
+	tex += u'%s\n'
+	tex += u'\n'
+	tex += u'\\end{tabular}\n'
+	tex += u'}\n'
+
+	current_meds = emr.get_current_substance_intake (
+		include_inactive = False,
+		include_unapproved = False,
+		order_by = u'brand, substance'
+	)
+
+	# create lines
+	lines = []
+	for med in current_meds:
+
+		lines.append(u'%s & %s & %s %s & {\\scriptsize %s} \\\\ \n \\hline \n' % (
+			med['substance'],
+			gmTools.coalesce(med['strength'], u''),
+			gmTools.coalesce(med['brand'], u''),
+			med['preparation'],
+			gmTools.coalesce(med['notes'], u'')
+		))
+
+	return tex % u' \n'.join(lines)
+
+#------------------------------------------------------------
+def format_substance_intake(emr=None, output_format=u'latex', table_type=u'by-brand'):
+
+	tex =  u'\\noindent %s {\\tiny (%s)\\par}\n' % (_('Medication list'), _('ordered by brand'))
+	tex += u'\n'
+	tex += u'\\noindent \\begin{tabular}{|l|l|}\n'
+	tex += u'\\hline\n'
+	tex += u'%s & %s \\\\ \n' % (_('Drug'), _('Regimen'))
+	tex += u'\\hline\n'
+	tex += u'\n'
+	tex += u'\\hline\n'
+	tex += u'%s\n'
+	tex += u'\n'
+	tex += u'\\end{tabular}\n'
+
+	current_meds = emr.get_current_substance_intake (
+		include_inactive = False,
+		include_unapproved = False,
+		order_by = u'brand, substance'
+	)
+
+	# aggregate data
+	line_data = {}
+	for med in current_meds:
+		identifier = gmTools.coalesce(med['brand'], med['substance'])
+
+		try:
+			line_data[identifier]
+		except KeyError:
+			line_data[identifier] = {'brand': u'', 'preparation': u'', 'schedule': u'', 'aims': [], 'strengths': []}
+
+		line_data[identifier]['brand'] = identifier
+		if med['strength'] is not None:
+			line_data[identifier]['strengths'].append(med['strength'].strip())
+		line_data[identifier]['preparation'] = med['preparation']
+		line_data[identifier]['schedule'] = gmTools.coalesce(med['schedule'], u'')
+		if med['aim'] not in line_data[identifier]['aims']:
+			line_data[identifier]['aims'].append(med['aim'])
+
+	# create lines
+	already_seen = []
+	lines = []
+	line1_template = u'%s %s & %s \\\\'
+	line2_template = u' & {\\scriptsize %s\\par} \\\\'
+
+	for med in current_meds:
+		identifier = gmTools.coalesce(med['brand'], med['substance'])
+
+		if identifier in already_seen:
+			continue
+
+		already_seen.append(identifier)
+
+		lines.append (line1_template % (
+			line_data[identifier]['brand'],
+			line_data[identifier]['preparation'],
+			line_data[identifier]['schedule']
+		))
+
+		strengths = u'/'.join(line_data[identifier]['strengths'])
+		if strengths == u'':
+			template = u' & {\\scriptsize %s\\par} \\\\'
+			for aim in line_data[identifier]['aims']:
+				lines.append(template % aim)
+		else:
+			if len(line_data[identifier]['aims']) == 0:
+				template = u'%s & \\\\'
+				lines.append(template % strengths)
+			else:
+				template = u'%s & {\\scriptsize %s\\par} \\\\'
+				lines.append(template % (strengths, line_data[identifier]['aims'][0]))
+				template = u' & {\\scriptsize %s\\par} \\\\'
+				for aim in line_data[identifier]['aims'][1:]:
+					lines.append(template % aim)
+
+		lines.append(u'\\hline')
+
+	return tex % u' \n'.join(lines)
 #============================================================
 class cBrandedDrug(gmBusinessDBObject.cBusinessDBObject):
 	"""Represents a drug as marketed by a manufacturer."""
@@ -929,6 +1074,14 @@ class cBrandedDrug(gmBusinessDBObject.cBusinessDBObject):
 
 	components = property(_get_components, lambda x:x)
 	#--------------------------------------------------------
+	def _get_is_vaccine(self):
+		cmd = u'SELECT EXISTS (SELECT 1 FROM clin.vaccine WHERE fk_brand = %(fk_brand)s)'
+		args = {'fk_brand': self._payload[self._idx['pk']]}
+		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = False)
+		return rows[0][0]
+
+	is_vaccine = property(_get_is_vaccine, lambda x:x)
+	#--------------------------------------------------------
 	def add_component(self, substance=None, atc=None):
 
 		# normalize atc
