@@ -113,6 +113,7 @@ class cProcedureEAPnl(wxgProcedureEAPnl.wxgProcedureEAPnl, gmEditArea.cGenericEd
 	#----------------------------------------------------------------
 	def __init_ui(self):
 		self._PRW_hospital_stay.add_callback_on_lose_focus(callback = self._on_hospital_stay_lost_focus)
+		self._PRW_hospital_stay.set_context(context = 'pat', val = gmPerson.gmCurrentPatient().ID)
 		self._PRW_location.add_callback_on_lose_focus(callback = self._on_location_lost_focus)
 
 		# location
@@ -843,11 +844,11 @@ class cEncounterEditAreaPnl(wxgEncounterEditAreaPnl.wxgEncounterEditAreaPnl):
 		self._PRW_encounter_type.SetBackgroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_WINDOW))
 		self._PRW_encounter_type.Refresh()
 
-		if not self._PRW_start.is_valid_timestamp():
+		if (not self._PRW_start.is_valid_timestamp()) or (self._PRW_start.GetValue().strip() == u''):
 			self._PRW_start.SetFocus()
 			return False
 
-		if not self._PRW_end.is_valid_timestamp():
+		if (not self._PRW_end.is_valid_timestamp()) or (self._PRW_end.GetValue().strip() == u''):
 			self._PRW_end.SetFocus()
 			return False
 
