@@ -10,7 +10,6 @@ import logging, datetime as pydt
 
 
 import wx
-#import wx.lib.inspection
 
 
 from Gnumed.pycommon import gmDispatcher, gmExceptions
@@ -40,7 +39,7 @@ class cXxxEAPnl(wxgXxxEAPnl.wxgXxxEAPnl, gmEditArea.cGenericEditAreaMixin):
 		except KeyError:
 			data = None
 
-		wxgXxxEAPnl.wxgXxxPatientEAPnl.__init__(self, *args, **kwargs)
+		wxgXxxEAPnl.wxgXxxEAPnl.__init__(self, *args, **kwargs)
 		gmEditArea.cGenericEditAreaMixin.__init__(self)
 
 		# Code using this mixin should set mode and data
@@ -114,7 +113,11 @@ class cXxxEAPnl(wxgXxxEAPnl.wxgXxxEAPnl, gmEditArea.cGenericEditAreaMixin):
 		if mode == 'edit':
 			if self.__data is None:
 				raise ValueError('[%s] <mode> "edit" needs data value' % self.__class__.__name__)
+
+		prev_mode = self.__mode
 		self.__mode = mode
+		if mode != prev_mode:
+			self.refresh()
 
 	mode = property(_get_mode, _set_mode)
 	#----------------------------------------------------------------
@@ -234,8 +237,6 @@ class cGenericEditAreaDlg2(wxgGenericEditAreaDlg2.wxgGenericEditAreaDlg2):
 		self.Refresh()
 
 		self._PNL_ea.refresh()
-
-#		wx.lib.inspection.InspectionTool().Show()
 	#--------------------------------------------------------
 	def _adjust_clear_revert_buttons(self):
 		if self._PNL_ea.data is None:
