@@ -5,7 +5,7 @@ __author__ = "Karsten Hilbert <Karsten.Hilbert@gmx.net>"
 __license__ = "GPL"
 
 
-import sys, logging, datetime as pyDT, decimal, os, webbrowser
+import sys, logging, datetime as pyDT, decimal, os, webbrowser, subprocess, codecs
 
 
 import wx, wx.grid, wx.lib.hyperlink
@@ -13,9 +13,10 @@ import wx, wx.grid, wx.lib.hyperlink
 
 if __name__ == '__main__':
 	sys.path.insert(0, '../../')
-from Gnumed.business import gmPerson, gmPathLab, gmSurgery, gmLOINC
+from Gnumed.business import gmPerson, gmPathLab, gmSurgery, gmLOINC, gmForms
 from Gnumed.pycommon import gmTools, gmDispatcher, gmMatchProvider, gmDateTime, gmI18N, gmCfg, gmShellAPI
-from Gnumed.wxpython import gmRegetMixin, gmPhraseWheel, gmEditArea, gmGuiHelpers, gmListWidgets, gmAuthWidgets, gmPatSearchWidgets
+from Gnumed.wxpython import gmRegetMixin, gmPhraseWheel, gmEditArea, gmGuiHelpers, gmListWidgets
+from Gnumed.wxpython import gmAuthWidgets, gmPatSearchWidgets, gmFormWidgets
 from Gnumed.wxGladeWidgets import wxgMeasurementsPnl, wxgMeasurementsReviewDlg
 from Gnumed.wxGladeWidgets import wxgMeasurementEditAreaPnl
 
@@ -105,17 +106,41 @@ def call_browser_on_measurement_type(measurement_type=None):
 		autoraise = True
 	)
 #----------------------------------------------------------------
-def edit_measurement(parent=None, measurement=None):
+def edit_measurement(parent=None, measurement=None, single_entry=False):
 	ea = cMeasurementEditAreaPnl(parent = parent, id = -1)
 	ea.data = measurement
 	ea.mode = gmTools.coalesce(measurement, 'new', 'edit')
-	dlg = gmEditArea.cGenericEditAreaDlg2(parent = parent, id = -1, edit_area = ea)
+	dlg = gmEditArea.cGenericEditAreaDlg2(parent = parent, id = -1, edit_area = ea, single_entry = single_entry)
 	dlg.SetTitle(gmTools.coalesce(measurement, _('Adding new measurement'), _('Editing measurement')))
 	if dlg.ShowModal() == wx.ID_OK:
 		dlg.Destroy()
 		return True
 	dlg.Destroy()
 	return False
+#================================================================
+def plot_measurements(parent=None, tests=None):
+
+	template = gmFormWidgets.manage_form_templates (
+		parent = parent,
+		active_only = True,
+		template_types = [u'gnuplot script']
+	)
+
+	if template is None:
+		gmGuiHelpers.gm_show_error (
+			aMessage = _('Cannot plot without a plot script.'),
+			aTitle = _('Plotting test results')
+		)
+		return False
+
+	fname_data = gmPathLab.export_results_for_gnuplot(results = tests)
+
+	script = template.instantiate()
+	script.data_filename = fname_data
+	script.generate_output(format = 'wxp') 		# Gnuplot output terminal
+#	if cleanup:
+#		script.cleanup()
+
 #================================================================
 #from Gnumed.wxGladeWidgets import wxgPrimaryCareVitalsInputPnl
 
@@ -264,6 +289,20 @@ class cMeasurementsGrid(wx.grid.Grid):
 
 		dlg.Destroy()
 	#------------------------------------------------------------
+	def plot_current_selection(self):
+
+		if not self.IsSelection():
+			gmDispatcher.send(signal = u'statustext', msg = _('Cannot plot results. No results selected.'))
+			return True
+
+		tests = self.__cells_to_data (
+			cells = self.get_selected_cells(),
+			exclude_multi_cells = False,
+			auto_include_multi_cells = True
+		)
+
+		plot_measurements(parent = self, tests = tests)
+	#------------------------------------------------------------
 	def get_selected_cells(self):
 
 		sel_block_top_left = self.GetSelectionBlockTopLeft()
@@ -826,7 +865,7 @@ class cMeasurementsGrid(wx.grid.Grid):
 	def __resize_corner_window(self, evt):
 		self.__WIN_corner.Layout()
 	#------------------------------------------------------------
-	def __cells_to_data(self, cells=None, exclude_multi_cells=False):
+	def __cells_to_data(self, cells=None, exclude_multi_cells=False, auto_include_multi_cells=False):
 		"""List of <cells> must be in row / col order."""
 		data = []
 		for row, col in cells:
@@ -844,11 +883,13 @@ class cMeasurementsGrid(wx.grid.Grid):
 				gmDispatcher.send(signal = u'statustext', msg = _('Excluding multi-result field from further processing.'))
 				continue
 
-			data_to_include = self.__get_choices_from_multi_cell(cell_data = data_list)
-
-			if data_to_include is None:
+			if auto_include_multi_cells:
+				data.extend(data_list)
 				continue
 
+			data_to_include = self.__get_choices_from_multi_cell(cell_data = data_list)
+			if data_to_include is None:
+				continue
 			data.extend(data_to_include)
 
 		return data
@@ -903,7 +944,7 @@ class cMeasurementsGrid(wx.grid.Grid):
 		if data is None:
 			return
 
-		edit_measurement(parent = self, measurement = data)
+		edit_measurement(parent = self, measurement = data, single_entry = True)
 	#------------------------------------------------------------
 #     def OnMouseMotionRowLabel(self, evt):
 #         x, y = self.CalcUnscrolledPosition(evt.GetPosition())
@@ -1013,6 +1054,9 @@ class cMeasurementsPnl(wxgMeasurementsPnl.wxgMeasurementsPnl, gmRegetMixin.cRege
 	def __on_sign_current_selection(self, evt):
 		self.data_grid.sign_current_selection()
 	#--------------------------------------------------------
+	def __on_plot_current_selection(self, evt):
+		self.data_grid.plot_current_selection()
+	#--------------------------------------------------------
 	def __on_delete_current_selection(self, evt):
 		self.data_grid.delete_current_selection()
 	#--------------------------------------------------------
@@ -1025,6 +1069,10 @@ class cMeasurementsPnl(wxgMeasurementsPnl.wxgMeasurementsPnl, gmRegetMixin.cRege
 		self.__action_button_popup.AppendItem(wx.MenuItem(self.__action_button_popup, menu_id, _('Review and &sign')))
 		wx.EVT_MENU(self.__action_button_popup, menu_id, self.__on_sign_current_selection)
 
+		menu_id = wx.NewId()
+		self.__action_button_popup.AppendItem(wx.MenuItem(self.__action_button_popup, menu_id, _('Plot')))
+		wx.EVT_MENU(self.__action_button_popup, menu_id, self.__on_plot_current_selection)
+
 		menu_id = wx.NewId()
 		self.__action_button_popup.AppendItem(wx.MenuItem(self.__action_button_popup, menu_id, _('Export to &file')))
 		#wx.EVT_MENU(self.__action_button_popup, menu_id, self.data_grid.current_selection_to_file)
@@ -1122,6 +1170,8 @@ class cMeasurementEditAreaPnl(wxgMeasurementEditAreaPnl.wxgMeasurementEditAreaPn
 	#--------------------------------------------------------
 	def _refresh_as_new(self):
 		self._PRW_test.SetText(u'', None, True)
+		self.__refresh_loinc_info()
+		self.__update_units_context()
 		self._TCTRL_result.SetValue(u'')
 		self._PRW_units.SetText(u'', None, True)
 		self._PRW_abnormality_indicator.SetText(u'', None, True)
@@ -1151,6 +1201,8 @@ class cMeasurementEditAreaPnl(wxgMeasurementEditAreaPnl.wxgMeasurementEditAreaPn
 	#--------------------------------------------------------
 	def _refresh_from_existing(self):
 		self._PRW_test.SetData(data = self.data['pk_test_type'])
+		self.__refresh_loinc_info()
+		self.__update_units_context()
 		self._TCTRL_result.SetValue(self.data['unified_val'])
 		self._PRW_units.SetText(self.data['val_unit'], self.data['val_unit'], True)
 		self._PRW_abnormality_indicator.SetText (
@@ -1183,6 +1235,8 @@ class cMeasurementEditAreaPnl(wxgMeasurementEditAreaPnl.wxgMeasurementEditAreaPn
 		self._refresh_from_existing()
 
 		self._PRW_test.SetText(u'', None, True)
+		self.__refresh_loinc_info()
+		self.__update_units_context()
 		self._TCTRL_result.SetValue(u'')
 		self._PRW_units.SetText(u'', None, True)
 		self._PRW_abnormality_indicator.SetText(u'', None, True)
@@ -1279,7 +1333,7 @@ class cMeasurementEditAreaPnl(wxgMeasurementEditAreaPnl.wxgMeasurementEditAreaPn
 				lab = None,
 				abbrev = self._PRW_test.GetValue().strip(),
 				name = self._PRW_test.GetValue().strip(),
-				unit = gmTools.none_if(self._PRW_units.GetValue().strip(), u'')
+				unit = gmTools.coalesce(self._PRW_units.GetData(), self._PRW_units.GetValue()).strip()
 			)
 			pk_type = tt['pk_test_type']
 
@@ -1298,18 +1352,25 @@ class cMeasurementEditAreaPnl(wxgMeasurementEditAreaPnl.wxgMeasurementEditAreaPn
 			('abnormality_indicator', self._PRW_abnormality_indicator),
 			('note_test_org', self._TCTRL_note_test_org),
 			('comment', self._TCTRL_narrative),
-			('val_normal_min', self._TCTRL_normal_min),
-			('val_normal_max', self._TCTRL_normal_max),
 			('val_normal_range', self._TCTRL_normal_range),
-			('val_target_min', self._TCTRL_target_min),
-			('val_target_max', self._TCTRL_target_max),
 			('val_target_range', self._TCTRL_target_range),
 			('norm_ref_group', self._TCTRL_norm_ref_group)
 		]
+		for field, widget in ctrls:
+			tr[field] = widget.GetValue().strip()
+
+		ctrls = [
+			('val_normal_min', self._TCTRL_normal_min),
+			('val_normal_max', self._TCTRL_normal_max),
+			('val_target_min', self._TCTRL_target_min),
+			('val_target_max', self._TCTRL_target_max)
+		]
 		for field, widget in ctrls:
 			val = widget.GetValue().strip()
-			if val != u'':
-				tr[field] = val
+			if val == u'':
+				tr[field] = None
+			else:
+				tr[field] = decimal.Decimal(val.replace(',', u'.', 1))
 
 		tr.save_payload()
 
@@ -1352,25 +1413,32 @@ class cMeasurementEditAreaPnl(wxgMeasurementEditAreaPnl.wxgMeasurementEditAreaPn
 		tr['pk_intended_reviewer'] = self._PRW_intended_reviewer.GetData()
 		tr['val_num'] = v_num
 		tr['val_alpha'] = v_al
-		tr['val_unit'] = self._PRW_units.GetValue().strip()
+		tr['val_unit'] = gmTools.coalesce(self._PRW_units.GetData(), self._PRW_units.GetValue()).strip()
 		tr['clin_when'] = self._DPRW_evaluated.GetData().get_pydt()
-		tr['abnormality_indicator'] = self._PRW_abnormality_indicator.GetValue().strip()
 
 		ctrls = [
+			('abnormality_indicator', self._PRW_abnormality_indicator),
 			('note_test_org', self._TCTRL_note_test_org),
 			('comment', self._TCTRL_narrative),
-			('val_normal_min', self._TCTRL_normal_min),
-			('val_normal_max', self._TCTRL_normal_max),
 			('val_normal_range', self._TCTRL_normal_range),
-			('val_target_min', self._TCTRL_target_min),
-			('val_target_max', self._TCTRL_target_max),
 			('val_target_range', self._TCTRL_target_range),
 			('norm_ref_group', self._TCTRL_norm_ref_group)
 		]
+		for field, widget in ctrls:
+			tr[field] = widget.GetValue().strip()
+
+		ctrls = [
+			('val_normal_min', self._TCTRL_normal_min),
+			('val_normal_max', self._TCTRL_normal_max),
+			('val_target_min', self._TCTRL_target_min),
+			('val_target_max', self._TCTRL_target_max)
+		]
 		for field, widget in ctrls:
 			val = widget.GetValue().strip()
-			if val != u'':
-				tr[field] = val
+			if val == u'':
+				tr[field] = None
+			else:
+				tr[field] = decimal.Decimal(val.replace(',', u'.', 1))
 
 		tr.save_payload()
 
@@ -1391,12 +1459,8 @@ class cMeasurementEditAreaPnl(wxgMeasurementEditAreaPnl.wxgMeasurementEditAreaPn
 		self._PRW_abnormality_indicator.add_callback_on_lose_focus(self._on_leave_indicator_prw)
 	#--------------------------------------------------------
 	def _on_leave_test_prw(self):
-		pk_type = self._PRW_test.GetData()
-		# units context
-		if pk_type is None:
-			self._PRW_units.unset_context(context = u'pk_type')
-		else:
-			self._PRW_units.set_context(context = u'pk_type', val = pk_type)
+		self.__refresh_loinc_info()
+		self.__update_units_context()
 	#--------------------------------------------------------
 	def _on_leave_indicator_prw(self):
 		# if the user hasn't explicitly enabled reviewing
@@ -1424,6 +1488,48 @@ class cMeasurementEditAreaPnl(wxgMeasurementEditAreaPnl.wxgMeasurementEditAreaPn
 		search_term = search_term.replace(' ', u'+')
 
 		call_browser_on_measurement_type(measurement_type = search_term)
+	#--------------------------------------------------------
+	# internal helpers
+	#--------------------------------------------------------
+	def __update_units_context(self):
+
+		self._PRW_units.unset_context(context = u'loinc')
+
+		tt = self._PRW_test.GetData(as_instance = True)
+
+		if tt is None:
+			self._PRW_units.unset_context(context = u'pk_type')
+			if self._PRW_test.GetValue().strip() == u'':
+				self._PRW_units.unset_context(context = u'test_name')
+			else:
+				self._PRW_units.set_context(context = u'test_name', val = self._PRW_test.GetValue().strip())
+			return
+
+		self._PRW_units.set_context(context = u'pk_type', val = tt['pk_test_type'])
+		self._PRW_units.set_context(context = u'test_name', val = tt['name'])
+
+		if tt['loinc'] is None:
+			return
+
+		self._PRW_units.set_context(context = u'loinc', val = tt['loinc'])
+	#--------------------------------------------------------
+	def __refresh_loinc_info(self):
+
+		self._TCTRL_loinc.SetValue(u'')
+
+		if self._PRW_test.GetData() is None:
+			return
+
+		tt = self._PRW_test.GetData(as_instance = True)
+
+		if tt['loinc'] is None:
+			return
+
+		info = gmLOINC.loinc2info(loinc = tt['loinc'])
+		if len(info) == 0:
+			return
+
+		self._TCTRL_loinc.SetValue(u'%s: %s' % (tt['loinc'], info[0]))
 #================================================================
 # measurement type handling
 #================================================================
@@ -1577,6 +1683,12 @@ limit 50""" % {'in_house': _('in house lab')}
 		self.matcher = mp
 		self.SetToolTipString(_('Select the type of measurement.'))
 		self.selection_only = False
+	#------------------------------------------------------------
+	def _data2instance(self):
+		if self.data is None:
+			return None
+
+		return gmPathLab.cMeasurementType(aPK_obj = self.data)
 #----------------------------------------------------------------
 class cMeasurementOrgPhraseWheel(gmPhraseWheel.cPhraseWheel):
 
@@ -1650,6 +1762,7 @@ limit 50"""
 		mp.setThresholds(1, 2, 4)
 		self._PRW_name.matcher = mp
 		self._PRW_name.selection_only = False
+		self._PRW_name.add_callback_on_lose_focus(callback = self._on_name_lost_focus)
 
 		# abbreviation
 		query = u"""
@@ -1667,19 +1780,6 @@ limit 50"""
 		self._PRW_abbrev.selection_only = False
 
 		# unit
-		# FIXME: use units from test_result
-		query = u"""
-select distinct on (conversion_unit)
-	conversion_unit,
-	conversion_unit
-from clin.test_type
-where
-	conversion_unit %(fragment_condition)s
-order by conversion_unit
-limit 50"""
-		mp = gmMatchProvider.cMatchProvider_SQL2(queries=query)
-		mp.setThresholds(1, 2, 3)
-		self._PRW_conversion_unit.matcher = mp
 		self._PRW_conversion_unit.selection_only = False
 
 		# loinc
@@ -1743,29 +1843,27 @@ limit 50"""
 		self._PRW_loinc.matcher = mp
 		self._PRW_loinc.selection_only = False
 		self._PRW_loinc.add_callback_on_lose_focus(callback = self._on_loinc_lost_focus)
+	#----------------------------------------------------------------
+	def _on_name_lost_focus(self):
 
-#		# test org
-#		query = u"""
-#select distinct on (internal_name)
-#	pk,
-#	internal_name
-#from clin.test_org
-#where
-#	internal_name %(fragment_condition)s
-#order by internal_name
-#limit 50"""
-#		mp = gmMatchProvider.cMatchProvider_SQL2(queries=query)
-#		mp.setThresholds(1, 2, 4)
-#		self._PRW_test_org.matcher = mp
-#		self._PRW_test_org.selection_only = False
+		test = self._PRW_name.GetValue().strip()
+
+		if test == u'':
+			self._PRW_conversion_unit.unset_context(context = u'test_name')
+			return
+
+		self._PRW_conversion_unit.set_context(context = u'test_name', val = test)
 	#----------------------------------------------------------------
 	def _on_loinc_lost_focus(self):
 		loinc = self._PRW_loinc.GetData()
 
 		if loinc is None:
 			self._TCTRL_loinc_info.SetValue(u'')
+			self._PRW_conversion_unit.unset_context(context = u'loinc')
 			return
 
+		self._PRW_conversion_unit.set_context(context = u'loinc', val = loinc)
+
 		info = gmLOINC.loinc2info(loinc = loinc)
 		if len(info) == 0:
 			self._TCTRL_loinc_info.SetValue(u'')
@@ -1801,9 +1899,12 @@ limit 50"""
 			lab = pk_org,
 			abbrev = self._PRW_abbrev.GetValue().strip(),
 			name = self._PRW_name.GetValue().strip(),
-			unit = gmTools.none_if(self._PRW_conversion_unit.GetValue().strip(), u'')
+			unit = gmTools.coalesce (
+				self._PRW_conversion_unit.GetData(),
+				self._PRW_conversion_unit.GetValue()
+			).strip()
 		)
-		tt['loinc'] = gmTools.none_if(self._PRW_loinc.GetValue().strip(), u'')
+		tt['loinc'] = gmTools.none_if(self._PRW_loinc.GetData().strip(), u'')
 		tt['comment_type'] = gmTools.none_if(self._TCTRL_comment_type.GetValue().strip(), u'')
 		tt.save()
 
@@ -1823,8 +1924,11 @@ limit 50"""
 		self.data['pk_test_org'] = pk_org
 		self.data['abbrev'] = self._PRW_abbrev.GetValue().strip()
 		self.data['name'] = self._PRW_name.GetValue().strip()
-		self.data['conversion_unit'] = gmTools.none_if(self._PRW_conversion_unit.GetValue().strip(), u'')
-		self.data['loinc'] = gmTools.none_if(self._PRW_loinc.GetValue().strip(), u'')
+		self.data['conversion_unit'] = gmTools.coalesce (
+			self._PRW_conversion_unit.GetData(),
+			self._PRW_conversion_unit.GetValue()
+		).strip()
+		self.data['loinc'] = gmTools.none_if(self._PRW_loinc.GetData().strip(), u'')
 		self.data['comment_type'] = gmTools.none_if(self._TCTRL_comment_type.GetValue().strip(), u'')
 		self.data.save()
 
@@ -1832,16 +1936,18 @@ limit 50"""
 	#----------------------------------------------------------------
 	def _refresh_as_new(self):
 		self._PRW_name.SetText(u'', None, True)
+		self._on_name_lost_focus()
 		self._PRW_abbrev.SetText(u'', None, True)
 		self._PRW_conversion_unit.SetText(u'', None, True)
 		self._PRW_loinc.SetText(u'', None, True)
-		self._TCTRL_loinc_info.SetValue(u'')
+		self._on_loinc_lost_focus()
 		self._TCTRL_comment_type.SetValue(u'')
 		self._PRW_test_org.SetText(u'', None, True)
 		self._TCTRL_comment_org.SetValue(u'')
 	#----------------------------------------------------------------
 	def _refresh_from_existing(self):
 		self._PRW_name.SetText(self.data['name'], self.data['name'], True)
+		self._on_name_lost_focus()
 		self._PRW_abbrev.SetText(self.data['abbrev'], self.data['abbrev'], True)
 		self._PRW_conversion_unit.SetText (
 			gmTools.coalesce(self.data['conversion_unit'], u''),
@@ -1853,7 +1959,7 @@ limit 50"""
 			self.data['loinc'],
 			True
 		)
-		self._TCTRL_loinc_info.SetValue(u'')			# FIXME: properly set
+		self._on_loinc_lost_focus()
 		self._TCTRL_comment_type.SetValue(gmTools.coalesce(self.data['comment_type'], u''))
 		self._PRW_test_org.SetText (
 			gmTools.coalesce(self.data['pk_test_org'], u'', self.data['internal_name_org']),
@@ -1876,33 +1982,119 @@ class cUnitPhraseWheel(gmPhraseWheel.cPhraseWheel):
 	def __init__(self, *args, **kwargs):
 
 		query = u"""
-select distinct val_unit,
-	val_unit, val_unit
-from clin.v_test_results
-where
-	(
-		val_unit %(fragment_condition)s
-			or
-		conversion_unit %(fragment_condition)s
-	)
-	%(ctxt_test_name)s
-	%(ctxt_test_pk)s
-order by val_unit
-limit 25"""
+
+SELECT DISTINCT ON (data) data, unit FROM (
+
+	SELECT rank, data, unit FROM (
+
+		(
+		-- via clin.v_test_results.pk_type (for types already used in results)
+		SELECT
+			val_unit as data,
+			val_unit || ' (' || name_tt || ')' as unit,
+			1 as rank
+		FROM
+			clin.v_test_results
+		WHERE
+			(
+				val_unit %(fragment_condition)s
+					OR
+				conversion_unit %(fragment_condition)s
+			)
+			%(ctxt_type_pk)s
+			%(ctxt_test_name)s
+
+		) UNION ALL (
+
+		-- via clin.test_type (for types not yet used in results)
+		SELECT
+			conversion_unit as data,
+			conversion_unit || ' (' || name || ')' as unit,
+			2 as rank
+		FROM
+			clin.test_type
+		WHERE
+			conversion_unit %(fragment_condition)s
+			%(ctxt_ctt)s
+
+		) UNION ALL (
+
+		-- via ref.loinc.ipcc_units
+		SELECT
+			ipcc_units as data,
+			ipcc_units || ' (' || term || ')' as unit,
+			3 as rank
+		FROM
+			ref.loinc
+		WHERE
+			ipcc_units %(fragment_condition)s
+			%(ctxt_loinc)s
+			%(ctxt_loinc_term)s
+
+		) UNION ALL (
+
+		-- via ref.loinc.submitted_units
+		SELECT
+			submitted_units as data,
+			submitted_units || ' (' || term || ')'  as unit,
+			3 as rank
+		FROM
+			ref.loinc
+		WHERE
+			submitted_units %(fragment_condition)s
+			%(ctxt_loinc)s
+			%(ctxt_loinc_term)s
+
+		) UNION ALL (
+
+		-- via ref.loinc.example_units
+		SELECT
+			example_units as data,
+			example_units || ' (' || term || ')'  as unit,
+			3 as rank
+		FROM
+			ref.loinc
+		WHERE
+			example_units %(fragment_condition)s
+			%(ctxt_loinc)s
+			%(ctxt_loinc_term)s
+		)
+
+	) as all_matching_units
+
+	WHERE data IS NOT NULL
+	ORDER BY rank
+
+) as ranked_matching_units
+
+LIMIT 50"""
 
 		ctxt = {
-			'ctxt_test_name': {
-				'where_part': u'and %(test)s in (name_tt, name_meta, code_tt, abbrev_meta)',
-				'placeholder': u'test'
-			},
-			'ctxt_test_pk': {
-				'where_part': u'and pk_test_type = %(pk_type)s',
+			'ctxt_type_pk': {
+				'where_part': u'AND pk_test_type = %(pk_type)s',
 				'placeholder': u'pk_type'
+			},
+			'ctxt_test_name': {
+				'where_part': u'AND %(test_name)s IN (name_tt, name_meta, code_tt, abbrev_meta)',
+				'placeholder': u'test_name'
+			},
+			'ctxt_ctt': {
+				'where_part': u'AND %(test_name)s IN (name, code, abbrev)',
+				'placeholder': u'test_name'
+			},
+			'ctxt_loinc': {
+				'where_part': u'AND code = %(loinc)s',
+				'placeholder': u'loinc'
+			},
+			'ctxt_loinc_term': {
+				'where_part': u'AND term ~* %(test_name)s',
+				'placeholder': u'test_name'
 			}
 		}
 
 		mp = gmMatchProvider.cMatchProvider_SQL2(queries=query, context=ctxt)
 		mp.setThresholds(1, 2, 4)
+		#mp.print_queries = True
 		gmPhraseWheel.cPhraseWheel.__init__ (
 			self,
 			*args,
@@ -1911,7 +2103,7 @@ limit 25"""
 		self.matcher = mp
 		self.SetToolTipString(_('Select the unit of the test result.'))
 		self.selection_only = False
-
+		self.phrase_separators = u'[;|]+'
 #================================================================
 
 #================================================================
