@@ -117,7 +117,7 @@ class cPatientPicture(wx.StaticBitmap):
 			gmGuiHelpers.gm_show_error (
 				aMessage = _(
 					'No image could be acquired from the source.\n\n'
-					'This may mean the scanner driver is not properly installed\n\n'
+					'This may mean the scanner driver is not properly installed.\n\n'
 					'On Windows you must install the TWAIN Python module\n'
 					'while on Linux and MacOSX it is recommended to install\n'
 					'the XSane package.'
@@ -167,9 +167,19 @@ class cPatientPicture(wx.StaticBitmap):
 
 		if photo is None:
 			fname = None
+			self.SetToolTipString (_(
+				'Patient picture.\n'
+				'\n'
+				'Right-click for context menu.'
+			))
 #			gmDispatcher.send(signal='statustext', msg=_('Cannot get most recent patient photo from database.'))
 		else:
 			fname = photo.export_to_file()
+			self.SetToolTipString (_(
+				'Patient picture (%s).\n'
+				'\n'
+				'Right-click for context menu.'
+			) % photo['date_generated'].strftime('%b %Y'))
 
 		return self.__set_pic_from_file(fname)
 	#-----------------------------------------------------------------
@@ -187,6 +197,7 @@ class cPatientPicture(wx.StaticBitmap):
 		del img_data
 		self.SetBitmap(bmp_data)
 		self.__pic_name = fname
+
 		return True
 	#-----------------------------------------------------------------
 	def _on_post_patient_selection(self):
