@@ -187,8 +187,8 @@ class cMainTopPanel(wx.Panel):
 
 		# create patient picture
 		self.patient_picture = gmPatPicWidgets.cPatientPicture(self, -1)
-		tt = wx.ToolTip(_('Patient picture.\nRight-click for context menu.'))
-		self.patient_picture.SetToolTip(tt)
+#		tt = wx.ToolTip(_('Patient picture.\nRight-click for context menu.'))
+#		self.patient_picture.SetToolTip(tt)
 
 		# create main sizer
 		self.szr_main = wx.BoxSizer(wx.HORIZONTAL)
@@ -343,7 +343,7 @@ class cMainTopPanel(wx.Panel):
 		for allergy in emr.get_allergies():
 			# in field: "true" allergies only, not intolerances
 			if allergy['type'] == 'allergy':
-				tmp.append(allergy['descriptor'][:10] + gmTools.u_ellipsis)
+				tmp.append(allergy['descriptor'][:10].strip() + gmTools.u_ellipsis)
 			# in tooltip
 			if allergy['definite']:
 				certainty = _('definite')
